﻿using System.Data;
using System.Collections.Generic;
using Microsoft.SharePoint.Client;
namespace Telerik.Reporting
{
    class SharePointDataSource
    {
        /// <summary>
        /// Stores the list schema and data
        /// </summary>
        DataTable ListData { get; set; }

        /// <summary>
        /// Used by ObjectDataSource for retrieving the specified SP list data
        /// </summary>
        /// <param name="spFullUrl">SharePoint full URL</param>
        /// <param name="listName">List name</param>
        /// <param name="username">username</param>
        /// <param name="password">password</param>
        /// <param name="domain">domain</param>
        /// <returns>SharePoint List data as DataTable</returns>
        public DataTable GetListData(string spFullUrl, 
                                        string listName, 
                                        string username, 
                                        string password, 
                                        string domain)
        {
            var context = GetContext(spFullUrl, 
                                     username, 
                                     password, 
                                     domain);
            // build the CAML query to get ALL items
            var query = new CamlQuery();
            query.ViewXml = "";
            var list = context.Web.Lists.GetByTitle(listName);
            ListItemCollection items = list.GetItems(query);
            items.RetrieveItems().Retrieve();
            context.ExecuteQuery();

            ListData = new DataTable("ListData");

            var keys = new List<string>(InitializeSchema(items));
            FillDataTable(items, 
                          keys);

            context.Dispose();
            return ListData;
        }

        void FillDataTable(ListItemCollection items, 
                           List<string> keys)
        {
            foreach (ListItem item in items)
            {
                var rowValues = new object[keys.Count];
                var counter = 0;
                foreach (var key in keys)
                {
                    rowValues[counter++] = item.FieldValues[key];
                }
                ListData.LoadDataRow(rowValues, true);
            }
        }

        IEnumerable<string> InitializeSchema(ListItemCollection items)
        {
            var firstRow = items[0].FieldValues;
            foreach (var key in firstRow.Keys)
            {
                if (null != firstRow[key])
                {
                    ListData.Columns.Add(key, firstRow[key].GetType());
                    yield return key;
                }
            }
        }

        static ClientContext GetContext(string spFullUrl, 
                                        string username, 
                                        string password, 
                                        string domain)
        {
            var context = new ClientContext(spFullUrl);
            var credetentials = new System.Net.NetworkCredential(username,
                password,
                domain);
            context.Credentials = credetentials;
            return context;
        }
    }
}